import types


class MixinDynamicInheritance:
    def __init__(self, mixin_class: type, new_class_name: str) -> None:
        self.mixin_class = mixin_class
        self.new_class_name = new_class_name

    def __call__(self, existing_class: type) -> type:
        return types.new_class(
            self.new_class_name, (self.mixin_class, existing_class), {}
        )



from unittest.mock import MagicMock

import pytest
from sphinx.application import Sphinx
from sphinx.builders import Builder
from sphinx.registry import SphinxComponentRegistry

from sphinxcontrib.kasane import (
    TranslatorSetUp,
    new_translator_class_for_builder,
)
from sphinxcontrib.kasane.conditions import (
    BuilderCondition,
    BuilderFormatCondition,
)
from sphinxcontrib.kasane.inheritance import MixinDynamicInheritance


class TestTranslatorSetUp:
    @pytest.fixture
    def builder(self) -> Builder:
        return MagicMock(spec=Builder)

    @pytest.fixture
    def registry(self) -> SphinxComponentRegistry:
        return MagicMock(spec=SphinxComponentRegistry())

    @pytest.fixture
    def app(
        self, builder: Builder, registry: SphinxComponentRegistry
    ) -> Sphinx:
        app = MagicMock(spec=Sphinx)
        app.builder = builder
        app.registry = registry
        return app

    @pytest.fixture
    def inheritance(self) -> MixinDynamicInheritance:
        return MagicMock(spec=MixinDynamicInheritance(None, ""))

    @pytest.fixture
    def unsatisfied_condition(self) -> BuilderCondition:
        condition = MagicMock(spec=BuilderCondition)
        condition.is_satisfied_by.return_value = False
        return condition

    def test_condition_not_satisfied(
        self,
        app: Sphinx,
        inheritance: MixinDynamicInheritance,
        unsatisfied_condition: BuilderCondition,
    ) -> None:
        sut = TranslatorSetUp(inheritance, unsatisfied_condition)

        sut(app)

        app.set_translator.assert_not_called()  # type: ignore[attr-defined]
        unsatisfied_condition.is_satisfied_by.assert_called_once_with(
            app.builder
        )

    @pytest.fixture
    def satisfied_condition(self) -> BuilderCondition:
        condition = MagicMock(spec=BuilderCondition)
        condition.is_satisfied_by.return_value = True
        return condition

    def test_condition_satisfied(
        self,
        app: Sphinx,
        inheritance: MixinDynamicInheritance,
        satisfied_condition: BuilderCondition,
    ) -> None:
        sut = TranslatorSetUp(inheritance, satisfied_condition)

        sut(app)

        app.registry.get_translator_class.assert_called_once_with(app.builder)  # type: ignore[attr-defined]  # NOQA: E501
        inheritance.assert_called_once_with(
            app.registry.get_translator_class.return_value  # type: ignore[attr-defined]  # NOQA: E501
        )
        app.set_translator.assert_called_once_with(  # type: ignore[attr-defined]  # NOQA: E501
            app.builder.name, inheritance.return_value, override=True
        )
        satisfied_condition.is_satisfied_by.assert_called_once_with(
            app.builder
        )


class TestNewTranslatorClassForBuilder:
    def test_new_translator_class_for_builder(self) -> None:
        class AwesomeMixin: ...  # NOQA: E701

        actual = new_translator_class_for_builder(
            "html", AwesomeMixin, "AwesomeTranslator"
        )

        assert isinstance(actual, TranslatorSetUp)
        self.assert_inheritance(
            actual.inheritance,
            MixinDynamicInheritance(AwesomeMixin, "AwesomeTranslator"),
        )
        self.assert_condition(actual.condition, "html")

    def assert_inheritance(
        self,
        actual: MixinDynamicInheritance,
        expected: MixinDynamicInheritance,
    ) -> None:
        assert isinstance(actual, MixinDynamicInheritance)
        assert vars(actual) == vars(expected)

    def assert_condition(
        self, actual: BuilderFormatCondition, expected_format: str
    ) -> None:
        assert isinstance(actual, BuilderFormatCondition)
        assert actual.format == expected_format



from typing import Protocol

from sphinx.builders import Builder


class BuilderCondition(Protocol):
    def is_satisfied_by(self, builder: Builder) -> bool: ...


class BuilderFormatCondition:
    def __init__(self, format: str) -> None:
        self.format = format

    def is_satisfied_by(self, builder: Builder) -> bool:
        return builder.format == self.format



from unittest.mock import MagicMock

from sphinx.builders import Builder

from sphinxcontrib.kasane.conditions import BuilderFormatCondition


class TestBuilderFormatCondition:
    def test_satisfied(self):
        html_format_builder = MagicMock(spec=Builder)
        html_format_builder.format = "html"

        sut = BuilderFormatCondition("html")

        assert sut.is_satisfied_by(html_format_builder)

    def test_not_satisfied(self):
        not_html_format_builder = MagicMock(spec=Builder)
        not_html_format_builder.format = "text"

        sut = BuilderFormatCondition("html")

        assert not sut.is_satisfied_by(not_html_format_builder)



from sphinxcontrib.kasane.inheritance import MixinDynamicInheritance


class TestMixinDynamicInheritance:
    def test_create_new_class(self) -> None:
        class AwesomeMixin: ...  # NOQA: E701

        class SomeClass: ...  # NOQA: E701

        sut = MixinDynamicInheritance(AwesomeMixin, "AwesomeNewClass")
        actual = sut(SomeClass)

        assert actual.__name__ == "AwesomeNewClass"
        assert issubclass(actual, AwesomeMixin)
        assert issubclass(actual, SomeClass)



from __future__ import annotations

from sphinx.application import Sphinx

from sphinxcontrib.kasane.conditions import (
    BuilderCondition,
    BuilderFormatCondition,
)
from sphinxcontrib.kasane.inheritance import MixinDynamicInheritance

__version__ = "0.1.0"


class TranslatorSetUp:
    def __init__(
        self, inheritance: MixinDynamicInheritance, condition: BuilderCondition
    ) -> None:
        self.inheritance = inheritance
        self.condition = condition

    def __call__(self, app: Sphinx) -> None:
        if not self.condition.is_satisfied_by(app.builder):
            return

        builder = app.builder
        translator_class = app.registry.get_translator_class(builder)
        app.set_translator(
            builder.name, self.inheritance(translator_class), override=True
        )


def new_translator_class_for_builder(
    builder_format: str, mixin_class: type, new_class_name: str
):
    condition = BuilderFormatCondition(builder_format)
    inheritance = MixinDynamicInheritance(mixin_class, new_class_name)
    return TranslatorSetUp(inheritance, condition)
